@@ -1,10 +1,9 @@
 """Class for RESQML Genetic Boundary Feature (horizon) organizational objects."""
 
-from ._utils import (equivalent_extra_metadata, alias_for_attribute, extract_has_occurred_during,
-                     equivalent_chrono_pairs, create_xml_has_occurred_during)
-
 import resqpy.olio.uuid as bu
 import resqpy.olio.xml_et as rqet
+import resqpy.organize
+import resqpy.organize._utils as ou
 from resqpy.olio.base import BaseResqpy
 from resqpy.olio.xml_namespaces import curly_namespace as ns
 
@@ -13,14 +12,25 @@ class GeneticBoundaryFeature(BaseResqpy):
     """Class for RESQML Genetic Boundary Feature (horizon) organizational objects."""
 
     resqml_type = "GeneticBoundaryFeature"
-    feature_name = alias_for_attribute("title")
+    feature_name = ou.alias_for_attribute("title")
     valid_kinds = ('horizon', 'geobody boundary')
 
-    def __init__(self, parent_model, uuid = None, kind = None, feature_name = None, extra_metadata = None):
+    def __init__(self,
+                 parent_model,
+                 uuid = None,
+                 kind = None,
+                 feature_name = None,
+                 extra_metadata = None,
+                 originator = None,
+                 absolute_age = None):
         """Initialises a genetic boundary feature (horizon or geobody boundary) organisational object."""
         self.kind = kind
-        self.absolute_age = None  # (timestamp, year offset) pair, or None; todo: support setting from args
-        super().__init__(model = parent_model, uuid = uuid, title = feature_name, extra_metadata = extra_metadata)
+        self.absolute_age = absolute_age  # (timestamp, year offset) pair, or None;
+        super().__init__(model = parent_model,
+                         uuid = uuid,
+                         title = feature_name,
+                         originator = originator,
+                         extra_metadata = extra_metadata)
 
     def _load_from_xml(self):
         self.kind = rqet.find_tag_text(self.root, 'GeneticBoundaryKind')
@@ -35,7 +45,7 @@ class GeneticBoundaryFeature(BaseResqpy):
             return False
         if self is other or bu.matching_uuids(self.uuid, other.uuid):
             return True
-        if check_extra_metadata and not equivalent_extra_metadata(self, other):
+        if check_extra_metadata and not ou.equivalent_extra_metadata(self, other):
             return False
         return self.feature_name == other.feature_name and self.kind == other.kind and self.absolute_age == other.absolute_age
 
