@@ -171,11 +171,16 @@ def property_kind_and_facet_from_keyword(keyword):
     # elif lk == 'sal':    # todo: salinity; local property kind needed; various units possible in Nexus
     elif lk == 'livecell' or lk.startswith('act'):
         property_kind = 'active'  # local property kind, see RESQML (2.0.1) usage guide, section 11.17
-    elif len(lk) <= 8 and (lk[0] == 'i' or lk.startswith('reg') or lk.startswith('creg')):
+    elif len(lk) <= 8 and (lk.startswith('reg') or lk.startswith('creg')):
         property_kind = 'region initialization'  # local property kind, see RESQML (2.0.1) usage guide, section 11.18
-        if lk[0] == 'i':
-            facet_type = 'what'
-            facet = lk[1:].lower()
+    elif lk in ['iequil', 'ipvt', 'irelpm', 'irock', 'iregion', 'iwater', 'ireg']:
+        property_kind = 'region initialization'  # local property kind, see RESQML (2.0.1) usage guide, section 11.18
+        facet_type = 'what'
+        facet = lk[1:]
+        if facet == 'equil':
+            facet = 'equilibration'
+        elif facet == 'relpm':
+            facet = 'relative permeability'
     return property_kind, facet_type, facet
 
 
